"""
gramps-web-desktop

Run your local gramps family trees in a browser.
"""

import os
import runpy
import sys
import tempfile
import argparse

import gramps
from gramps.cli.clidbman import CLIDbManager
from gramps.gen.dbstate import DbState
from gramps.gen.db.utils import open_database

from ._version import __version__

CONFIG_TEMPLATE = """
TREE="{tree_name}"
SECRET_KEY="{secret_key}"
USER_DB_URI="sqlite:///{path_to_database}"
MEDIA_BASE_DIR="{media_base_dir}"
SEARCH_INDEX_DB_URI="sqlite:///{path_to_index_database}"
"""

def main(raw_args=sys.argv[1:]):
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter
    )
    parser.add_argument(
        "TREE",
        help="Name of Gramps family tree",
        type=str,
        nargs='?',
    )
    parser.add_argument(
        "USER",
        help="User name",
        type=str,
        nargs='?',
    )
    parser.add_argument(
        "PASSWORD",
        help="User's password",
        type=str,
        nargs='?',
    )
    parser.add_argument(
        "--version",
        help="Show version of gramps-web-desktop",
        action="store_const",
        const=True,
        default=False,
    )

    args, rest = parser.parse_known_args(raw_args)

    if args.version:
        print("gramps-web-desktop %s" % __version__)
        sys.exit()


    if "GRAMPS_RESOURCES" not in os.environ:
        os.environ["GRAMPS_RESOURCES"] = os.path.split(
            os.path.split(gramps.__file__)[0]
        )[0]

    HERE = os.path.abspath(os.path.dirname(__file__))

    os.environ["GRAMPSWEB_STATIC_PATH"] = os.path.abspath(os.path.join(HERE, "frontend"))

    cli = CLIDbManager(DbState())
    databases = {key: value for key,value in cli.family_tree_list()}

    tree_name = args.TREE
    if tree_name not in databases:
        print("Use one of the following as first argument for tree:")
        for key in databases:
            print("    %r" % key)
        sys.exit()

    if args.USER is None or args.PASSWORD is None:
        print("Requires user and password: gramps-web-desktop TREE USER PASSWORD")
        sys.exit()
    
    path_to_database = os.path.join(databases[tree_name], "sqlite.db")
    path_to_index_database = os.path.join(databases[tree_name], "index.db")
    secret_key = "my-secret-key"

    db = open_database(tree_name)
    if db is None:
        print("Database %r is locked" % tree_name)
        sys.exit()

    media_base_dir = db.get_mediapath()
    if media_base_dir:
        media_base_dir = media_base_dir.format(**os.environ)
    else:
        media_base_dir = ""
    db.close()

    config_contents = CONFIG_TEMPLATE.format(
        tree_name=tree_name,
        secret_key=secret_key,
        path_to_database=path_to_database,
        media_base_dir=media_base_dir,
        path_to_index_database=path_to_index_database,
    )
    with tempfile.NamedTemporaryFile("w", delete=False, suffix=".cfg", prefix="gramps-web-") as config_file:
        config_file.write(config_contents)

    print("GRAMPS_RESOURCES:", os.environ["GRAMPS_RESOURCES"])
    print("GRAMPSWEB_STATIC_PATH:", os.environ["GRAMPSWEB_STATIC_PATH"])
    print("MEDIA_BASE_DIR:", media_base_dir)
    print()
    print("Make sure you logout when finished")

    # First, make a user/password with admin role
    user_exists = False
    sys.argv = ["gramps_webapi.py", "--config", config_file.name, "user", "add", "--role", "5", args.USER, args.PASSWORD]
    try:
        runpy.run_module("gramps_webapi", run_name="__main__")
    except ValueError as exc:
        if exc.args[0] == "User already exists":
            user_exists = True
        else:
            raise

    # Next, run gramps-web-api:

    sys.argv = [
        "gramps_webapi.py",
        "--config", config_file.name,
        "run",
        "--open-browser", "tab",
        "--use-wsgi",
        "--debug-level", "warning"
    ]
    runpy.run_module("gramps_webapi", run_name="__main__")

    # Finally, delete user/password with admin role

    if not user_exists:
        sys.argv = ["gramps_webapi.py", "--config", config_file.name, "user", "delete", args.USER]
        runpy.run_module("gramps_webapi", run_name="__main__")



# -*- coding: utf-8 -*-
import io
import os

import setuptools

HERE = os.path.abspath(os.path.dirname(__file__))


def get_version(file, name="__version__"):
    """
    Get the version of the package from the given file by
    executing it and extracting the given `name`.
    """
    path = os.path.realpath(file)
    version_ns = {}
    with io.open(path, encoding="utf8") as f:
        exec(f.read(), {}, version_ns)
    return version_ns[name]


__version__ = get_version(os.path.join(HERE, "gramps_web_desktop/_version.py"))

with io.open(os.path.join(HERE, "README.md"), encoding="utf8") as fh:
    long_description = fh.read()

setup_args = dict(
    name="gramps-web-desktop",
    version=__version__,
    url="https://github.com/dsblank/gramps-web-desktop",
    author="",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        "gramps>=5.2.0",
        "gramps-webapi[ai]>=2.6.0",
    ],
    packages=[
        "gramps_web_desktop",
    ],
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "gramps-web-desktop = gramps_web_desktop:main",
            "gwd = gramps_web_desktop:main",
        ]
    },
    python_requires=">=3.7",
    license="AGPL-3.0 license",
    platforms="Linux, Mac OS X, Windows",
)

if __name__ == "__main__":
    setuptools.setup(**setup_args)
